/*
 * Copyright 2020 Ladifire. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import * as React from 'react';
import {SlotProvider, unwrapDOMRef, useDOMRef, useHasChild, useStyleProps, joinClasses} from '@ladifire-ui-react/utils';
import {CommonIcons} from '@ladifire-ui-react/icons';
import {DialogContext, DialogContextValue} from './context';
import {DismissButton} from '@react-ui-aria/overlays';
import {DOMRef} from '@react-ui-types/shared';
import {FocusScope} from '@react-ui-aria/focus';
import {Grid} from '@ladifire-ui-react/layout';
import {mergeProps} from '@react-ui-aria/utils';
import {SpectrumDialogProps} from '@react-ui-types/dialog';
import {useDialog} from '@react-aria/dialog';
import {Button} from '@ladifire-ui-react/button';

import stylex from "@ladifire/stylex";

const styles: any = stylex.create({
  hero: {
    gridArea: 'hero',
    borderTopLeftRadius: 8,
    borderTopRightRadius: 8,
    backgroundSize: 'cover',
    backgroundPosition: 'center center',
    overflow: 'hidden',
  },
  header: {
    gridArea: 'header',
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-end',
    boxSizing: 'border-box',
    outline: 'none',
  },
  heading: {
    gridArea: 'heading',
    margin: 0,
    display: 'flex',
    alignItems: 'center',
  },
  typeIcon: {
    gridArea: 'typeIcon',
  },
  divider: {
    gridArea: 'divider',
    width: '100%',
    marginTop: 4,
  },
  content: {
    gridArea: 'content',
    boxSizing: 'border-box',
    overflowY: 'auto',
    WebkitOverflowScrolling: 'touch',
    outline: 'none',
  },
  footerDivider: {
    gridArea: 'footerDivider',
    width: '100%',
    height: 1,
    marginTop: 0,
  },
  footer: {
    gridArea: 'footer',
    display: 'flex',
    flexWrap: 'wrap',
    outline: 'none',
  },
  buttonsGroup: {
    gridArea: 'buttonGroup',
    display: 'flex',
    justifyContent: 'flex-end',
    maxWidth: '100%',
    paddingTop: 16,
  },
  root: {
    display: 'flex',
    boxSizing: 'border-box',
    maxWidth: '100%',
    maxHeight: 'inherit',
    outline: 'none',
    width: '100%',
    minWidth: 288,
  },
  dismiss: {
    gridArea: 'closeButton',
    alignSelf: 'start',
    justifySelf: 'end',
    marginRight: '12px!important',
    marginTop: '8px!important',
  },
});

let sizeMap = {
  S: 'small',
  M: 'medium',
  L: 'large',
  fullscreen: 'fullscreen',
  fullscreenTakeover: 'fullscreenTakeover'
};

export interface DialogProps extends SpectrumDialogProps {
  scrolled?: boolean;
  headingClassName?: string;
  headerClassName?: string;
  heroClassName?: string;
  typeIconClassName?: string;
  dividerClassName?: string;
  contentClassName?: string;
  footerClassName?: string;
  buttonGroupClassName?: string;
}

function Dialog(props: DialogProps, ref: DOMRef) {
  let {
    type = 'modal',
    ...contextProps
  } = React.useContext(DialogContext) || {} as DialogContextValue;

  let {
    children,
    isDismissable = contextProps.isDismissable,
    onDismiss = contextProps.onClose,
    size,
    scrolled,
    headingClassName,
    headerClassName,
    heroClassName,
    typeIconClassName,
    dividerClassName,
    contentClassName,
    footerClassName,
    buttonGroupClassName,
    ...otherProps
  } = props;
  let {styleProps} = useStyleProps(otherProps);

  size = type === 'popover' ? (size || 'S') : (size || 'L');

  let domRef = useDOMRef(ref);
  let gridRef = React.useRef();
  let sizeVariant = sizeMap[type] || sizeMap[size];
  let {dialogProps, titleProps} = useDialog(mergeProps(contextProps, props), domRef);

  let hasHeader = useHasChild(`.dialog__heading`, unwrapDOMRef(gridRef));
  let hasFooter = useHasChild(`.dialog__footer`, unwrapDOMRef(gridRef));

  let slots = {
    hero: {UNSAFE_className: joinClasses(
        stylex(styles.hero),
        heroClassName,
      )},
    header: {UNSAFE_className: joinClasses(
        stylex(styles.header),
        'dialog__heading',
        headerClassName,
      )},
    heading: {UNSAFE_className: joinClasses(
        stylex(styles.heading),
        stylex.dedupe(
          !hasHeader ? {
            gridArea: 'heading-start / heading-start / header-end / header-end',
            paddingRight: 0,
          } : null,
        ),
        headingClassName,
      ),
      ...titleProps},
    typeIcon: {UNSAFE_className: joinClasses(
        stylex(styles.typeIcon),
        typeIconClassName,
      )},
    divider: {UNSAFE_className: joinClasses(
        stylex(styles.divider),
        dividerClassName,
      ), shadow: scrolled, size: 'M'},
    content: {UNSAFE_className: joinClasses(
        stylex(styles.content),
        contentClassName,
      )},
    footerDivider: {UNSAFE_className: stylex(styles.footerDivider)},
    footer: {UNSAFE_className: joinClasses(
        stylex(styles.footer),
        stylex.dedupe(
          isDismissable ? {
            gridArea: 'footer / footer/ buttonGroup / buttonGroup',
          } : null,
        ),
        'dialog__footer',
        footerClassName
      )},
    buttonGroup: {UNSAFE_className: joinClasses(
        stylex(styles.buttonsGroup),
        stylex.dedupe(
          !hasFooter ? {
            gridArea: 'footer-start / footer-start / buttonGroup-end / buttonGroup-end',
          } : null,
        ),
        buttonGroupClassName,
      ),
      align: 'end'}
  };

  // If rendered in a popover or tray there won't be a visible dismiss button,
  // so we render a hidden one for screen readers.
  let dismissButton: JSX.Element;
  if (type === 'popover' || type === 'tray') {
    dismissButton = <DismissButton onDismiss={onDismiss} />;
  }

  return (
    <FocusScope contain restoreFocus>
      <section
        {...styleProps}
        {...dialogProps}
        ref={domRef}
        className={joinClasses(
          stylex(styles.root),
          styleProps.className,
        )}
      >
        <Grid
          ref={gridRef}
          UNSAFE_className={stylex.dedupe(
            !isDismissable ? {
              display: 'grid',
              gridTemplateColumns: '12px auto 1fr auto minmax(0, auto) 12px',
              gridTemplateRows: 'auto 12px auto auto 1fr auto auto 12px',
              width: '100%',
              gridTemplateAreas: '"hero     hero        hero      hero          hero        hero"' +
                '    ".        .           .         .             .           ."' +
                '    ".        heading     header    header        typeIcon    ."' +
                '    "divider  divider     divider   divider       divider     divider"' +
                '    "content  content     content   content       content     content"' +
                '    "footerDivider  footerDivider   footerDivider   footerDivider       footerDivider       footerDivider"' +
                '    ".        footer      footer    buttonGroup   buttonGroup ."' +
                '    ".        .           .         .             .           ."'
            } : {
              display: 'grid',
              gridTemplateColumns: '12px auto 1fr auto minmax(0, auto) minmax(0, 40px) 12px',
              gridTemplateRows: 'auto 12px auto auto 1fr auto auto 12px',
              gridTemplateAreas: '"hero     hero      hero      hero          hero          hero          hero"' +
                '    ".        .         .         .             .             closeButton   closeButton"' +
                '    ".        heading   header    header        typeIcon      closeButton   closeButton"' +
                '    "divider  divider   divider   divider       divider       divider       divider"' +
                '    "content  content   content   content       content       content       content"' +
                '    "footerDivider  footerDivider   footerDivider   footerDivider       footerDivider       footerDivider       footerDivider"' +
                '    ".        footer    footer    buttonGroup   buttonGroup   buttonGroup   ."' +
                '    ".        .         .         .             .             .             ."',
              width: '100%',
            },
          )}
        >
          <SlotProvider slots={slots}>
            {children}
          </SlotProvider>
          {isDismissable &&
          <Button
            variant='secondary'
            square
            rounded
            className={stylex(styles.dismiss)}
            aria-label='dismiss'
            onPress={onDismiss}
          >
            <CommonIcons.ICloseIcon filter="secondary-icon"/>
          </Button>
          }
        </Grid>
        {dismissButton}
      </section>
    </FocusScope>
  );
}

/**
 * Dialogs are windows containing contextual information, tasks, or workflows that appear over the user interface.
 * Depending on the kind of Dialog, further interactions may be blocked until the Dialog is acknowledged.
 */
let _Dialog = React.forwardRef(Dialog);
export {_Dialog as Dialog};
