/*
 * Copyright 2020 Ladifire. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import * as React from 'react';
import {useDOMRef, useStyleProps, joinClasses} from '@ladifire-ui-react/utils';
import {DOMRef} from '@react-ui-types/shared';
import {mergeProps} from '@react-ui-aria/utils';
import {ModalProps} from '@react-ui-types/overlays';
import {useOverlay} from '@react-ui-aria/overlays';
import {Content, Footer, Header} from '@ladifire-ui-react/view';
import {Heading} from '@ladifire-ui-react/text';
import {Divider} from '@ladifire-ui-react/divider';
import {Overlay} from '@ladifire-ui-react/overlays';
import {ButtonGroup} from '@ladifire-ui-react/buttongroup';
import {Button} from '@ladifire-ui-react/button';

import stylex from "@ladifire/stylex";

import {Dialog} from './Dialog';
import {Underlay} from './Underlay';
import {
  WithModalProviderContext,
  WithModalProviderContextType,
} from './WithModalProvider';

type Style =
  | 'dialog'
  | 'marginRight'
  | 'root'
  | 'wrapper';

const styles = stylex.create<Style>({
  dialog: {
    paddingLeft: 16,
    paddingRight: 16,
    paddingTop: 16,
  },
  marginRight: {
    marginRight: 8,
  },
  root: {
    alignItems: 'inherit',
    maxHeight: 'inherit',
    flexGrow: 'inherit',
    flexShrink: 'inherit',
    maxWidth: 'inherit',
    position: 'relative',
    flexDirection: 'inherit',
    width: 'inherit',
    minHeight: 'inherit',
    display: 'inherit',
    justifyContent: 'inherit',
  },
  wrapper: {
    justifyContent: 'center',
    alignItems: 'flex-start',
    display: 'flex',
    pointerEvents: 'none',
  },
});

interface ModalWrapperProps extends React.HTMLAttributes<HTMLElement> {
  children: React.ReactNode,
  isOpen?: boolean,
  onClose?: () => void,
  onDismiss?: () => void,
  type?: 'fullscreen' | 'fullscreenTakeover',
  isDismissable?: boolean,
  scrollContent?: boolean,
  zIndex?: number,
  fixed?: boolean,
  showConfirm?: boolean,
}

function Modal(props: ModalProps, ref: DOMRef<HTMLDivElement>) {
  let {
    children,
    onClose,
    type,
    isDismissable,
    scrollContent,
    closeConfirm,
    confirmProps,
    onDismiss,
    showConfirm,
    onShowConfirmChange,
    ...otherProps
  } = props;

  let domRef = useDOMRef(ref);
  let {styleProps} = useStyleProps(props);

  const confirmRef = React.useRef();

  if (closeConfirm && (!confirmProps || !confirmProps.heading)) {
    throw new Error('When closeConfirm is true, confirmProps.heading, confirmProps.content is required');
  }

  const [confirm, setConfirm] = React.useState(false);
  React.useEffect(() => {
    setConfirm(showConfirm);
  }, [showConfirm]);

  React.useEffect(() => {
    if (typeof onShowConfirmChange === 'function') {
      onShowConfirmChange(confirm);
    }
  }, [confirm]);

  const handleCancelConfirm = () => {
    setConfirm(false);
  };

  const handleCloseConfirm = () => {
    setConfirm(false);
    onClose();
  };

  const handleClose = () => {
    if (closeConfirm) {
      setConfirm(true);
    } else {
      handleCloseConfirm();
    }
  };

  return (
    <Overlay
      {...otherProps}
    >
      <Underlay />
      <ModalWrapper
        {...styleProps}
        onClose={handleClose}
        scrollContent={scrollContent}
        type={type}
        isDismissable={isDismissable}
        ref={domRef}
      >
        {children}
      </ModalWrapper>
      {
        closeConfirm && confirm && (
          <div>
            <Underlay
              isOpen={true}
              zIndex={3}
            />
            <ModalWrapper
              {...styleProps}
              isOpen={confirm}
              fixed={confirm}
              ref={confirmRef}
            >
              <Dialog
                isDismissable={confirmProps.isDismissable}
                contentClassName={stylex(styles.dialog)}
                onDismiss={handleCancelConfirm}
              >
                <Heading>
                  {confirmProps.heading}
                </Heading>
                {Boolean(confirmProps.header) && (
                  <Header>{confirmProps.header}</Header>
                )}
                {confirmProps.showDivider && <Divider />}
                <Content>
                  {confirmProps.content}
                </Content>
                {Boolean(confirmProps.footer) && (
                  <Footer>{confirmProps.footer}</Footer>
                )}
                <ButtonGroup>
                  <Button
                    variant={confirmProps.cancelVariant || 'transparent'}
                    onPress={handleCancelConfirm}
                    className={stylex(styles.marginRight)}
                  >
                    {confirmProps.cancelText || 'Cancel'}
                  </Button>
                  <Button
                    variant={confirmProps.confirmVariant || 'secondary'}
                    onPress={handleCloseConfirm}
                  >
                    {confirmProps.confirmText || 'OK'}
                  </Button>
                </ButtonGroup>
              </Dialog>
            </ModalWrapper>
          </div>
        )
      }
    </Overlay>
  );
}

let ModalWrapper = React.forwardRef(function (props: ModalWrapperProps, ref: React.RefObject<HTMLDivElement>) {
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  let {
    children,
    isOpen,
    fixed = false,
    type,
    isDismissable,
    scrollContent,
    ...otherProps
  } = props;
  // let typeVariant = typeMap[type];

  const [modalProviderContext, setModalProviderContext] = React.useState<WithModalProviderContextType | null>(null);
  React.useEffect(() => {
    if (modalProviderContext &&
      modalProviderContext.onModalOpenChange &&
      typeof modalProviderContext.onModalOpenChange === 'function')
    {
      modalProviderContext.onModalOpenChange(isOpen);
    }
  }, [isOpen]);

  let {overlayProps} = useOverlay(props, ref);

  let wrapperClassName = stylex.dedupe(
    {
      display: 'flex',
      pointerEvents: 'none',
      alignItems: 'stretch',
      justifyContent: 'center',
      boxSizing: 'border-box',
      width: '100%',
      visibility: 'visible',
      transition: 'visibility 0ms linear var(--lf-duration-extra-extra-short-in)',
      paddingBottom: 'calc(var(--header-height) + 16px)',
      paddingTop: 'calc(var(--header-height) + 16px)',
      paddingLeft: 8,
      paddingRight: 8,
    },
    isOpen ? {
      overflow: 'visible',
    } : null,
    !scrollContent ? {
      minHeight: '100vh',
    } : null,
    fixed ? {
      position: 'fixed',
      top: 0,
      left: 0,
      bottom: 0,
      right: 0,
      zIndex: 4,
    } : {
      position: 'relative',
      zIndex: 2,
    },
  );

  let modalClassName = joinClasses(
    stylex.dedupe(
      {
        backgroundColor: 'var(--card-background)',
        boxShadow: '0 12px 28px 0 var(--shadow-2),0 2px 4px 0 var(--shadow-1),inset 0 0 0 1px var(--shadow-inset)',
        borderRadius: 8,
        pointerEvents: 'auto',
        maxWidth: '90vw',
      },
      isOpen ? {
        opacity: 1,
        visibility: 'visible',
      } : {
        opacity: 0,
        visibility: 'hidden',
      },
      scrollContent ? {
        maxHeight: '90vh',
      } : null,
    ),
    otherProps.className
  );

  return (
    <WithModalProviderContext.Consumer>
      {(_context: WithModalProviderContextType) => {
        if (!_context) {
          throw new Error('Modal should use with WithModalProvider wrapper.');
        }
        setModalProviderContext(_context);

        return (
          <div className={stylex(styles.root)}>
            <div className={wrapperClassName}>
              <div className={stylex(styles.wrapper)}>
                <div
                  {...mergeProps(otherProps, overlayProps)}
                  ref={ref}
                  className={modalClassName}
                  data-testid="modal"
                  data-ismodal={true}
                >
                  {children}
                </div>
              </div>
            </div>
          </div>
        );
      }}
    </WithModalProviderContext.Consumer>
  );
});

let _Modal = React.forwardRef(Modal);
export {_Modal as Modal};
