/*
 * Copyright 2020-present Ladifire. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import * as React from 'react';
import {DOMRef} from "@react-ui-types/shared";

export interface WithModalProviderContextType {
  onModalOpenChange: (isOpen: boolean) => void;
  closeAllModals: () => void;
  isModalOpen?: boolean;
}

export interface WithModalProviderProps {
  /** The content of the Provider. */
  children: React.ReactNode;
  className?: string;
  onModalOpenChange?: (isOpen: boolean) => void;
}

const Context = React.createContext<WithModalProviderContextType | null>(null);

export function useProvider(): WithModalProviderContextType {
  return React.useContext(Context);
}

function WithModalProvider(props: WithModalProviderProps, ref: DOMRef<HTMLDivElement>) {
  let {
    children,
    className,
    onModalOpenChange,
    ...otherProps
  } = props;

  const [isModalOpen, setIsModalOpen] = React.useState(false);
  React.useEffect(() => {
    if (typeof onModalOpenChange === 'function') {
      onModalOpenChange(isModalOpen);
    }
  }, [isModalOpen]);

  let _context = useProvider();
  const context: WithModalProviderContextType = Object.assign(_context || {}, {
    onModalOpenChange: (isOpen) => {
      setIsModalOpen(isOpen);
    },
    closeAllModals: () => setIsModalOpen(false),
    isModalOpen,
  });

  return (
    <Context.Provider value={context}>
      <div className={className}>
        {children}
      </div>
    </Context.Provider>
  );
}

/**
 * Provider is the container for all React Spectrum applications.
 * It defines the theme, locale, and other application level settings,
 * and can also be used to provide common properties to a group of components.
 */
let _WithModalProvider = React.forwardRef(WithModalProvider);
export {_WithModalProvider as WithModalProvider};
export {Context as WithModalProviderContext}
